﻿// SpringBuilder.as - Actionscript 3 Version - Jim Bumgardner 2008
//

package {
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.ui.Keyboard;

	public class SpringBuilder
	{
		public var SW,SH;

		
		public var gravK = 0.25;	// set this with slider
		public var stiffK = .25;
		public var frictionK = .02;
		public var showDots = true;

		var ss:SpringSim;
		var btnShowDotsController; // Set this up with btn_showDots
		var btnClrController;
		var btnPlayController = null;
		var btnBuildController = null;
		var btnDeleteController = null;
		var myLoader:URLLoader;
		var itsContainer;
		var spritesByName = {};

		public function SpringBuilder(parMC, btn_clear, btn_showdots, btn_curvy, btn_play, btn_build, btn_delete, slider1, slider2, slider3, slider4)
		{
			SW = parMC.stage.stageWidth;
			SH = parMC.stage.stageHeight;
			itsContainer = parMC;
			
			parMC.stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			parMC.stage.addEventListener(MouseEvent.MOUSE_DOWN, handleMouseDown);
			
			ss = new SpringSim(parMC, gravK, stiffK, frictionK, true);
			ss.simFloor = parMC.stage.stageHeight - 100;
			ss.setFloorRoughness(.25);
			ss.setRenderFlags(SpringSim.RENDER_SPRINGS | SpringSim.RENDER_DOTS | SpringSim.RENDER_CURVY);
			
			myLoader = new URLLoader(new URLRequest("lampncube.xml"));
			myLoader.addEventListener(Event.COMPLETE, xmlLoadHandler);

			btnClrController = new PlainButton(btn_clear, 'clear', clearFunc);
			btnShowDotsController = new ToggleButton(btn_showdots, 'show dots', true, showdotsToggle);
			new ToggleButton(btn_curvy, 'draw curvy', true, function(t) { ss.setDrawCurvy(t);} );
			
			new SliderController(slider1, 'gravity',-2,2, gravK, function (t) {	ss.setGravity(t); });
			new SliderController(slider2, 'stiffness',0,1, stiffK, function (t) {	ss.setStiffness(t); });
			new SliderController(slider3, 'air resistance',0,.5, frictionK, function (t) {	ss.setFriction(t); });
			new SliderController(slider4, 'ground friction',0,1, 1-ss.floorDamping,function(t) { ss.setFloorRoughness(t); } );

			btnPlayController = new SwitchButton(btn_play, 'simulate', true,   setMode, 0);
			btnBuildController = new SwitchButton(btn_build, 'build',   false,  setMode, 1);
			btnDeleteController = new SwitchButton(btn_delete, 'delete', false,  setMode, 2);
			
			setMode(0);
		}

		function handleKeyDown(evt:KeyboardEvent)
		{
			var code = evt.keyCode;
			if (code == Keyboard.BACKSPACE || code == Keyboard.DELETE)
			{
				if (ss.selectedDot)
					ss.deleteDot(ss.selectedDot);
			}
			var ch = String.fromCharCode(evt.charCode);
			trace('got char ' + ch);
			if (ch == 'n')
				ss.addAMouseDot();
			else if (ch == 'c')
				btnClrController.pressFunc();
			else if (ch == 'f') {
				if (ss.selectedDot != undefined)
					ss.selectedDot.setDotProps(!ss.selectedDot.isAnchor, true);
			}
			else if (ch == ' ')
				setMode(ss.editMode == 0? 1 : 0);
			else if (ch == 'd')
				btnShowDotsController.onRelease();
		}
		
		function handleMouseDown(evt)
		{
		  trace("got mouse down");
		  if (ss.editMode == 1 && !ss.isOverDot) {
			  ss.addAMouseDot();
		  }
		}
		
		function clearFunc()
		{
			ss.reset();
			var mc = ss.addDotMC(SW/2, 10, true);
			
			ss.drawSprings();
			setMode(1);
			ss.doSelectDot(mc);
			btnShowDotsController.setState(true);
		}

		function showdotsToggle(f)
		{
			showDots = f;
			ss.showDots(showDots);
		}
		
		function setMode(editMode)
		{
			trace("got setmode " + editMode);
			if (btnPlayController)
				btnPlayController.setState(editMode == 0);
			if (btnBuildController)
				btnBuildController.setState(editMode == 1);
			if (btnDeleteController)
				btnDeleteController.setState(editMode == 2);
			ss.setEditMode(editMode);
		}
		
		function xmlLoadHandler(evt)
		{
			layerNbr = 100;
			ox = 0;
			oy = 200;
			tw = 1;
			th = -1;
			ss.reset();
			loadModel();
		}

		function loadModel()
		{
			trace('loading model');
		    var myXML = XML(myLoader.data);
			if (myXML.hasOwnProperty("settings")) {
				if (myXML.settings.@gravitydirection == 'up') {
					ss.setGravity(-ss.gravK);
				}
				if (myXML.settings.@wavedirection == 'forward') {
				}
				if (myXML.settings.@autoreverse == 'on')
					ss.setAutoReverse(true);
				else
					ss.setAutoReverse(false);
			}
			if (myXML.hasOwnProperty("gridscale")) 
			{
				tw = Number(myXML.gridscale.@cellwidth);
				th = Number(myXML.gridscale.@cellheight);
				ox = Number(myXML.gridscale.@originx);
				oy = Number(myXML.gridscale.@originy);
			}
			if (myXML.hasOwnProperty("environment")) {
				if (myXML.environment.@gravity)
					ss.setGravity(Number(myXML.environment.@gravity));
				if (myXML.environment.@friction)
					ss.setFriction(Number(myXML.environment.@friction));
				if (myXML.environment.@springyness)
					ss.setStiffness(Number(myXML.environment.@springyness));
			}
			if (myXML.hasOwnProperty("collisions")) {
				if (myXML.collisions.@surface_friction)
					ss.setFloorRoughness(Number(myXML.collisions.@surface_friction));
			}
			if (myXML.hasOwnProperty("wave")) {
				var amp = Number(myXML.wave.@amplitude);
				var phase = Number(myXML.wave.@phase)*Math.PI*2;
				var speed = Number(myXML.wave.@speed);
				ss.setWave(amp,phase,speed);
			}
			for each (var poly:XML in myXML..poly)
			{
				var clr = poly.@color;
				if (clr.charAt(0) == '#')
					clr = Number('0x' + clr.substr(1));
				var ptNames = poly.@points.split(',');
				var curvy = Number(poly.@curvy);
				var pts = [];
				for (var i = 0; i < ptNames.length; ++i) 
				{
					pts.push(spritesByName[ptNames[i]]);
				}
				ss.addPolygon(pts,clr,curvy);
			}
			for each (var mass:XML in myXML..mass)
			{
				var mc = ss.addDotMC(ox+tw*Number(mass.@x), oy+th*Number(mass.@y), (Number(mass.@fixed) == 1) );
				spritesByName[mass.@id] = mc;
			}
			for each (var spring:XML in myXML..spring) // also 'muscle'
			{
				var a = spring.@a;
				var b = spring.@b;
		
				var spr = ss.addSpring(spritesByName[a], spritesByName[b]);
				
				if (spring.@restlength > 0)
				{
					spr.restLen = spring.@restlength;
				}
				if (spring.@amplitude > 0) {
					amp = Number(spring.@amplitude);
					phase = Number(spring.@phase)*Math.PI*2;
					spr.setMuscle(amp,phase);
				}
			}
		}

		
	} // end class
} // end package

