﻿// SpringBuilder.as
//
// Spring physics simulation
//
//
// - Jim Bumgardner

#include "SpringSim.as"
#include "jbumUI.as"

SW = Stage.width;
SH = Stage.height;

gravK = 0.25;	// set this with slider
stiffK = .25;
frictionK = .02;

showDots = true;
layerCtr = 10;

// This determines the dot number - it reuses the slots occupied by
// deleted dots (which are undefined)

this.onKeyDown = function()
{
	var code = Key.getCode();
	if (code == Key.BACKSPACE || code == Key.DELETEKEY)
	{
		if (ss.selectedDot)
			ss.deleteDot(ss.selectedDot);
	}
	var ch = String.fromCharCode(Key.getAscii());
	if (ch == 'n')
		ss.addAMouseDot(layerCtr++);
	else if (ch == 'c')
		btn_clr.onRelease();
	else if (ch == 'f') {
		if (ss.selectedDot != undefined)
			ss.selectedDot.setDotProps(!ss.selectedDot.isAnchor, true);
	}
	else if (ch == ' ')
		setMode(ss.editMode == 0? 1 : 0);
	else if (ch == 'd')
		bt_showdots.onRelease();

}

this.onMouseDown = function()
{
  if (ss.editMode == 1 && !ss.isOverDot) {
	  ss.addAMouseDot(layerCtr++);
  }
}

function clearFunc()
{
	for (var i = myDots.length-1; i >= 0; --i) {
		myDots[i].removeMovieClip();
	}
	myDots = [];

	ss.reset();

	var mc = ss.addDotMC(SW/2, 10, true, layerCtr++);
	
	ss.drawSprings();
	setMode(1);
	ss.doSelectDot(mc);
	btn_showdots.setState(true);
}

function showdotsToggle(f)
{
	showDots = f;
	ss.showDots(showDots);
}

function setMode(editMode)
{
	btn_play.setState(editMode == 0);
	btn_build.setState(editMode == 1);
	btn_delete.setState(editMode == 2);
	ss.setEditMode(editMode);
}

// One time initialization

Key.addListener(this);

ss = new SpringSim(_root, gravK, stiffK, frictionK, true);
ss.simFloor = 300;
ss.setFloorRoughness(.25);
ss.setRenderFlags(SpringSim.RENDER_SPRINGS | SpringSim.RENDER_DOTS | SpringSim.RENDER_CURVY);


// XML Reader
myDots = [];
XMLnode.prototype.loadModel = function()
{
	if (this.nodeName == 'settings') {
		if (this.attributes['gravitydirection'] == 'up') {
			ss.setGravity(-ss.gravK);
		}
		if (this.attributes['wavedirection'] == 'forward') {
		}
		if (this.attributes['autoreverse'] == 'on')
			ss.setAutoReverse(true);
		else
			ss.setAutoReverse(false);
	}
	if (this.nodeName == 'gridscale') 
	{
		tw = Number(this.attributes['cellwidth']);
		th = Number(this.attributes['cellheight']);
		ox = Number(this.attributes['originx']);
		oy = Number(this.attributes['originy']);
	}
	if (this.nodeName == 'environment') {
		if (this.attributes['gravity'])
			ss.setGravity(Number(this.attributes['gravity']));
		if (this.attributes['friction'])
			ss.setFriction(Number(this.attributes['friction']));
		if (this.attributes['springyness'])
			ss.setStiffness(Number(this.attributes['springyness']));
	}
	else if (this.nodeName == 'poly') {
		var clr = this.attributes['color'];
		if (clr.charAt(0) == '#')
			clr = Number('0x' + clr.substr(1));
		var ptNames = this.attributes['points'].split(',');
		var curvy = Number(this.attributes['curvy']);
		var pts = [];
		for (var i = 0; i < ptNames.length; ++i) 
		{
			pts.push(_root[ptNames[i]]);
		}
		ss.addPolygon(pts,clr,curvy);
	}
	else if (this.nodeName == 'collisions') {
		if (this.attributes['surface_friction'])
			ss.setFloorRoughness(Number(this.attributes['surface_friction']));
	}
	else if (this.nodeName == 'wave') {
		var amp = Number(this.attributes['amplitude']);
		var phase = Number(this.attributes['phase'])*Math.PI*2;
		var speed = Number(this.attributes['speed']);
		ss.setWave(amp,phase,speed);
	}
	else if (this.nodeName == 'mass') {
		// id,x,y,vx,vy
		var id = this.attributes['id'];
		var mc = _root.createEmptyMovieClip(id, _root.layerNbr++);
		myDots.push(mc);
		mc._x = ox+tw*Number(this.attributes['x']);
		mc._y = oy+th*Number(this.attributes['y']);
		mc.vx = Number(this.attributes['vx']);
		mc.vy = Number(this.attributes['vx']);
		var fixed = (Number(this.attributes['fixed']) == 1);
		mc.setDotProps(fixed, true);
		mc.drawDot();
	}
	else if (this.nodeName == 'spring' || this.nodeName == 'muscle') {
		// id,x,y,vx,vy
		var a = this.attributes['a'];
		var b = this.attributes['b'];
		var restLen = this.attributes['restlength'];
		var amp = Number(this.attributes['amplitude']);
		var phase = Number(this.attributes['phase'])*Math.PI*2;

		var spr = ss.addSpring(_root[a], _root[b]);
		
		if (restLen != undefined)
			spr.restLen = restLen;
		if (amp != undefined && phase != undefined)
			spr.setMuscle(amp,phase);
	}
	if (this.hasChildNodes()) {
		this.firstChild.loadModel();
	}
	if (this.nextSibling != undefined) {
		this.nextSibling.loadModel();
	}
}

var myxml = new XML();

myxml.onLoad = function()
{
	_root.layerNbr = 100;
	ox = 0;
	oy = 200;
	tw = 1;
	th = -1;
	ss.reset();
	for (var i = myDots.length-1; i >= 0; --i) {
		myDots[i].removeMovieClip();
	}
	myDots = [];
	// trace("doctype = " + this.docTypeDecl);
	this.loadModel();
	setupUI();
	
}

myxml.load("lampncube.xml");

btn_clear.setupButton('clear', clearFunc);
btn_showdots.setupToggleButton('show dots', true, showdotsToggle);
btn_curvy.setupToggleButton('draw curvy', true, function(t) { ss.setDrawCurvy(t);} );

btn_play.setupRadioButton('simulate', true,   setMode, 0);
btn_build.setupRadioButton('build',   false,  setMode, 1);
btn_delete.setupRadioButton('delete', false,  setMode, 2);

slider1_mc.setupSlider('gravity',-2,2, gravK, function (t) {	ss.setGravity(t); });
slider2_mc.setupSlider('stiffness',0,1, stiffK, function (t) {	ss.setStiffness(t); });
slider3_mc.setupSlider('air resistance',0,.5, frictionK, function (t) {	ss.setFriction(t); });
slider4_mc.setupSlider('ground friction',0,1, 1-ss.floorDamping,function(t) { ss.setFloorRoughness(t); } );

setMode(0);

